@@ -76,12 +76,13 @@ class Feature(TypedDict, total=False):
     bbox: Optional[List[float]]
 
 
-class FeatureCollection(TypedDict, total=False):
-    """Simple FeatureCollection model."""
+class ItemList(TypedDict):
+    """Items."""
 
-    type: str
-    features: List[Feature]
-    bbox: Optional[List[float]]
+    items: List[Feature]
+    matched: Optional[int]
+    next: Optional[int]
+    prev: Optional[int]
 
 
 class Column(BaseModel):
@@ -144,7 +145,7 @@ class Column(BaseModel):
     @property
     def is_datetime(self) -> bool:
         """Returns true if this property is a datetime column."""
-        return self.type in ("timestamp", "timestamptz")
+        return self.type in ("timestamp", "timestamptz", "date")
 
 
 class Parameter(Column):
@@ -527,7 +528,7 @@ class Collection(BaseModel):
         if datetime:
             if not self.datetime_columns:
                 raise MissingDatetimeColumn(
-                    "Must have timestamp typed column to filter with datetime."
+                    "Must have timestamp/timestamptz/date typed column to filter with datetime."
                 )
 
             datetime_column = self.get_datetime_column(dt)
@@ -739,8 +740,11 @@ class Collection(BaseModel):
         simplify: Optional[float] = None,
         geom_as_wkt: bool = False,
         function_parameters: Optional[Dict[str, str]] = None,
-    ) -> Tuple[FeatureCollection, int]:
+    ) -> ItemList:
         """Build and run Pg query."""
+        limit = limit or features_settings.default_features_limit
+        offset = offset or 0
+
         function_parameters = function_parameters or {}
 
         if geom and geom.lower() != "none" and not self.get_geometry_column(geom):
@@ -751,7 +755,7 @@ class Collection(BaseModel):
                 f"Limit can not be set higher than the `tipg_max_features_per_query` setting of {features_settings.max_features_per_query}"
             )
 
-        count = await self._features_count_query(
+        matched = await self._features_count_query(
             pool=pool,
             ids_filter=ids_filter,
             datetime_filter=datetime_filter,
@@ -784,10 +788,13 @@ class Collection(BaseModel):
                 function_parameters=function_parameters,
             )
         ]
+        returned = len(features)
 
-        return (
-            FeatureCollection(type="FeatureCollection", features=features),
-            count,
+        return ItemList(
+            items=features,
+            matched=matched,
+            next=offset + returned if matched - returned > offset else None,
+            prev=max(offset - limit, 0) if offset else None,
         )
 
     async def get_tile(
@@ -876,8 +883,17 @@ class Collection(BaseModel):
         return {**geoms, **props}
 
 
+class CollectionList(TypedDict):
+    """Collections."""
+
+    collections: List[Collection]
+    matched: Optional[int]
+    next: Optional[int]
+    prev: Optional[int]
+
+
 class Catalog(TypedDict):
-    """Collection Catalog."""
+    """Internal Collection Catalog."""
 
     collections: Dict[str, Collection]
     last_updated: datetime.datetime
@@ -893,6 +909,8 @@ async def get_collection_index(  # noqa: C901
     exclude_functions: Optional[List[str]] = None,
     exclude_function_schemas: Optional[List[str]] = None,
     spatial: bool = True,
+    spatial_extent: bool = True,
+    datetime_extent: bool = True,
 ) -> Catalog:
     """Fetch Table and Functions index."""
     schemas = schemas or ["public"]
@@ -906,7 +924,9 @@ async def get_collection_index(  # noqa: C901
             :functions,
             :exclude_functions,
             :exclude_function_schemas,
-            :spatial
+            :spatial,
+            :spatial_extent,
+            :datetime_extent
         );
     """  # noqa: W605
 
@@ -921,6 +941,8 @@ async def get_collection_index(  # noqa: C901
             exclude_functions=exclude_functions,
             exclude_function_schemas=exclude_function_schemas,
             spatial=spatial,
+            spatial_extent=spatial_extent,
+            datetime_extent=datetime_extent,
         )
 
         catalog: Dict[str, Collection] = {}
@@ -956,7 +978,7 @@ async def get_collection_index(  # noqa: C901
             geometry_column = None
 
             for c in properties:
-                if c.get("type") in ("timestamp", "timestamptz"):
+                if c.get("type") in ("timestamp", "timestamptz", "date"):
                     if (
                         table_conf.get("datetimecol") == c["name"]
                         or datetime_column is None
